/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor.name.container;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.swt.SWT;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.DropTarget;
import org.eclipse.swt.dnd.DropTargetListener;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.ui.forms.widgets.TableWrapData;
import org.eclipse.ui.forms.widgets.TableWrapLayout;

import eu.etaxonomy.cdm.model.common.CdmBase;
import eu.etaxonomy.taxeditor.editor.CdmDataTransfer;
import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.editor.IDropTargetable;
import eu.etaxonomy.taxeditor.editor.name.TaxonNameEditor;
import eu.etaxonomy.taxeditor.editor.name.dnd.NameEditorDropTargetEffect;
import eu.etaxonomy.taxeditor.editor.name.dnd.NameEditorDropTargetListener;
import eu.etaxonomy.taxeditor.preference.Resources;

/**
 * <p>Abstract AbstractGroup class.</p>
 *
 * @author p.ciardelli
 * @author n.hoffmann
 * @created 25.06.2008
 * @version 1.0
 */
public abstract class AbstractGroup implements IDropTargetable{

	private Composite control;

	private final List<AbstractGroupedContainer> groupedContainers = new ArrayList<AbstractGroupedContainer>();

	protected TaxonNameEditor editor;

	private DropTarget target;
	private DropTargetListener dropListener;

	/**
	 * <p>Constructor for AbstractGroup.</p>
	 *
	 * @param editor a {@link eu.etaxonomy.taxeditor.editor.name.TaxonNameEditor} object.
	 * @param <T> a T object.
	 */
	public AbstractGroup(TaxonNameEditor editor){
		this.editor = editor;
	}

	protected void createContent(){
		createControl();

		createContainers();

		// Drop functionality
		this.setDroppable(true);

		editor.getManagedForm().reflow(true);
	}

	/**
	 * <p>createControl</p>
	 */
	protected void createControl() {
		control = editor.getToolkit().createComposite(editor.getControl());

		control.setLayoutData(new TableWrapData(TableWrapData.FILL_GRAB));
		TableWrapLayout layout = new TableWrapLayout();

		layout.topMargin = 0;
		layout.rightMargin = 0;
		layout.bottomMargin = 1;
		layout.leftMargin = 0;

		layout.verticalSpacing = 0;
		layout.horizontalSpacing = 0;
		control.setLayout(layout);

		control.setBackground(Display.getDefault().getSystemColor(SWT.COLOR_GRAY));
	}

	/**
	 * <p>add</p>
	 *
	 * @param groupedContainer a {@link eu.etaxonomy.taxeditor.editor.name.container.AbstractGroupedContainer} object.
	 */
	public void add(AbstractGroupedContainer groupedContainer){
		groupedContainers.add(groupedContainer);
		groupedContainer.setGroup(this);
	}

	/**
	 * <p>remove</p>
	 *
	 * @param groupedContainer a {@link eu.etaxonomy.taxeditor.editor.name.container.AbstractGroupedContainer} object.
	 */
	public void remove(AbstractGroupedContainer groupedContainer){
		groupedContainer.dispose();
		groupedContainers.remove(groupedContainer);

		if(!(this instanceof AcceptedGroup) && groupedContainers.isEmpty()){
			getEditor().removeGroup(this);
		}
	}

	/**
	 * <p>Getter for the field <code>groupedContainers</code>.</p>
	 *
	 * @return a {@link java.util.List} object.
	 */
	public List<AbstractGroupedContainer> getGroupedContainers(){
		return groupedContainers;
	}


	/**
	 * <p>setDroppable</p>
	 *
	 * @param droppable a boolean.
	 */
	public void setDroppable(boolean droppable) {
		if (droppable) {
			Transfer[] types = new Transfer[] {CdmDataTransfer.getInstance()};
			int operations = DND.DROP_MOVE;
			target = new DropTarget(control, operations);
			target.setTransfer(types);
			dropListener = new NameEditorDropTargetListener(this);
			target.addDropListener(dropListener);

			target.setDropTargetEffect(new NameEditorDropTargetEffect(control));

		} else {
			if (dropListener != null) {
				target.removeDropListener(dropListener);
			}
		}
	}

	/**
	 * <p>getNameEditor</p>
	 *
	 * @return a {@link eu.etaxonomy.taxeditor.editor.name.TaxonNameEditor} object.
	 */
	@Override
    public TaxonNameEditor getEditor() {
		return editor;
	}

	/**
	 * <p>dragEntered</p>
	 */
	@Override
    public void dragEntered() {
		Color color = EditorUtil.getColor(Resources.COLOR_DRAG_ENTER);

		for(AbstractGroupedContainer container : groupedContainers){
			container.setBackground(color);
		}
	}

	/**
	 * <p>dragLeft</p>
	 */
	@Override
    public void dragLeft() {
		for(AbstractGroupedContainer container : groupedContainers){
			container.restoreColor();
		}
	}

	/** {@inheritDoc} */
	@Override
    public boolean postOperation(CdmBase objectAffectedByOperation) {
		editor.getMultiPageTaxonEditor().changed(objectAffectedByOperation);
		redraw();
		return true;
	}

	/**
	 * <p>redraw</p>
	 */
	public void redraw(){
		emptyGroup();
		createContainers();
	}

	protected abstract void createContainers();

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.editor.IDropTargetable#getControl()
	 */
	@Override
	public Composite getControl() {
		return control;
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.operation.IPostOperationEnabled#onComplete()
	 */
	@Override
	public boolean onComplete() {
		return true;
	}

	public void dispose(){
		if(getControl() != null){
			for(AbstractGroupedContainer container : getGroupedContainers()){
				container.dispose();
			}
			getControl().setMenu(null);
			getControl().dispose();
		}
	}

	/**
	 * @param object
	 */
	public void setMenu(Menu menu) {
		getControl().setMenu(menu);
		for(AbstractGroupedContainer container : getGroupedContainers()){
			container.setMenu(menu);
		}

	}

	protected void emptyGroup(){
		for(AbstractGroupedContainer container : groupedContainers){
			container.dispose();
		}
		groupedContainers.clear();
	}
}
