/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor.name.container;

import org.eclipse.jface.text.source.Annotation;
import org.eclipse.jface.text.source.AnnotationPainter.SquigglesStrategy;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Point;

/**
 * Adds ability to draw multiline squiggles when a <code>StyledText</code>
 * contains line-wrapping.
 *
 * @author p.ciardelli
 * @created 21.11.2008
 * @version 1.0
 */
public class LineWrapSquigglesStrategy extends SquigglesStrategy {
	
	/** Constant <code>ID="linewrap_squigglesstrategy"</code> */
	public static final String ID = "linewrap_squigglesstrategy"; //$NON-NLS-1$

	private GC gc;

	private Color color;

	private int lineHeight;

	private int baseline;

	private StyledText textWidget;

	private int offset;
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.text.source.AnnotationPainter$SquigglesStrategy#draw(org.eclipse.jface.text.source.Annotation, org.eclipse.swt.graphics.GC, org.eclipse.swt.custom.StyledText, int, int, org.eclipse.swt.graphics.Color)
	 */
	/** {@inheritDoc} */
	public void draw(Annotation annotation, GC gc, StyledText textWidget, int offset, int length, Color color) {
		
		this.gc = gc;
		this.color = color;
		this.textWidget = textWidget;
		this.offset = offset;
		
		if (gc != null) {

			if (length < 1)
				return;
			
			baseline = textWidget.getBaseline(offset);
			lineHeight = textWidget.getLineHeight(offset);
			
			Point right = null;
			int offsetNewline = offset;
			int end = offset + length;
						
			// Go through the length one character at a time
			for (int i = offset; i <= end; i++) {
				
				// If the y of the current offset is different from that of the last offset,
				//	we are on a new line
				if (right != null && textWidget.getLocationAtOffset(i).y > right.y) {
										
					// Draw a line of squigglies
					drawPolyline(offsetNewline, right);

					// Save offset of line break
					offsetNewline = i;
				}
								
				// Get x,y position in case the next char is on a new line
				right = textWidget.getLocationAtOffset(i);					
					
			}
			
			// Draw the last line of squigglies 
			drawPolyline(offsetNewline, right);

		} else {
			textWidget.redrawRange(offset, length, true);
		}
	}
	
	/**
	 * Draws a squiggly line from the offset <code>offsetNewline</code> to
	 * the x,y coordinates at <code>right</code>.
	 * 
	 * @param offsetNewline
	 * @param right
	 */
	private void drawPolyline(int offsetNewline, Point right) {

		// Get offset at last line break
		Point left = textWidget.getLocationAtOffset(offsetNewline);
		
		// Prevent solitary red dot from appearing at EOL
		if (left.equals(right)) {
			return;
		}
		
		// Only start drawing from 0 if not on the first line
		if (offsetNewline != offset) {
			
			// Offset.x is at the end of the first letter of the new line, not at 0
			left.x = 0;
		}
		
		int[] polyline= computePolyline(left, right, baseline, lineHeight);

		gc.setLineWidth(0); // NOTE: 0 means width is 1 but with optimized performance
		gc.setLineStyle(SWT.LINE_SOLID);
		gc.setForeground(color);
		gc.drawPolyline(polyline);
	}

	/**
	 * Copied verbatim from {@link org.eclipse.jface.text.source.AnnotationPainter$SquigglesStrategy} 
	 * 
	 * @see org.eclipse.jface.text.source.AnnotationPainter$SquigglesStrategy
	 *  
	 * @param left
	 * @param right
	 * @param baseline
	 * @param lineHeight
	 * @return
	 */
	private int[] computePolyline(Point left, Point right, int baseline, int lineHeight) {

		final int WIDTH= 4; // must be even
		final int HEIGHT= 2; // can be any number

		int peaks= (right.x - left.x) / WIDTH;
		if (peaks == 0 && right.x - left.x > 2)
			peaks= 1;

		int leftX= left.x;

		// compute (number of point) * 2
		int length= ((2 * peaks) + 1) * 2;
		if (length < 0)
			return new int[0];

		int[] coordinates= new int[length];

		// cache peeks' y-coordinates
		int top= left.y + Math.min(baseline + 1, lineHeight - HEIGHT - 1);
		int bottom= top + HEIGHT;

		// populate array with peek coordinates
		for (int i= 0; i < peaks; i++) {
			int index= 4 * i;
			coordinates[index]= leftX + (WIDTH * i);
			coordinates[index+1]= bottom;
			coordinates[index+2]= coordinates[index] + WIDTH/2;
			coordinates[index+3]= top;
		}

		// the last down flank is missing
		coordinates[length-2]= Math.min(Math.max(0, right.x - 1), left.x + (WIDTH * peaks));
		coordinates[length-1]= bottom;

		return coordinates;
	}
}
