package eu.etaxonomy.taxeditor.editor.view.derivate;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.UUID;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.util.LocalSelectionTransfer;
import org.eclipse.jface.viewers.AbstractTreeViewer;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TreeNode;
import org.eclipse.jface.viewers.TreeSelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.IMemento;
import org.eclipse.ui.ISelectionListener;
import org.eclipse.ui.ISelectionService;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.part.EditorPart;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.api.conversation.IConversationEnabled;
import eu.etaxonomy.cdm.api.service.IOccurrenceService;
import eu.etaxonomy.cdm.hibernate.HibernateProxyHelper;
import eu.etaxonomy.cdm.model.common.CdmBase;
import eu.etaxonomy.cdm.model.molecular.SingleRead;
import eu.etaxonomy.cdm.model.occurrence.FieldUnit;
import eu.etaxonomy.cdm.model.occurrence.SpecimenOrObservationBase;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.cdm.model.taxon.TaxonNode;
import eu.etaxonomy.cdm.persistence.hibernate.CdmDataChangeMap;
import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.editor.MultiPageTaxonEditor;
import eu.etaxonomy.taxeditor.editor.l10n.Messages;
import eu.etaxonomy.taxeditor.editor.view.derivate.searchFilter.DerivateSearchCompositeController;
import eu.etaxonomy.taxeditor.model.IContextListener;
import eu.etaxonomy.taxeditor.model.IDirtyMarkable;
import eu.etaxonomy.taxeditor.model.IPartContentHasDetails;
import eu.etaxonomy.taxeditor.model.IPartContentHasFactualData;
import eu.etaxonomy.taxeditor.model.IPartContentHasMedia;
import eu.etaxonomy.taxeditor.model.IPartContentHasSupplementalData;
import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.operation.IPostOperationEnabled;
import eu.etaxonomy.taxeditor.session.ICdmEntitySession;
import eu.etaxonomy.taxeditor.session.ICdmEntitySessionEnabled;
import eu.etaxonomy.taxeditor.store.CdmStore;
import eu.etaxonomy.taxeditor.view.derivateSearch.DerivateContentProvider;
import eu.etaxonomy.taxeditor.view.derivateSearch.DerivateLabelProvider;

/**
 * Displays the derivate hierarchy of the specimen specified in the editor input.
 *
 */
public class DerivateView extends EditorPart implements IPartContentHasFactualData, IConversationEnabled,
        ICdmEntitySessionEnabled, IDirtyMarkable, IPostOperationEnabled, IPartContentHasDetails, IPartContentHasSupplementalData, IPartContentHasMedia,
        IContextListener, ISelectionListener {
    private static final String SPECIMEN_EDITOR = Messages.DerivateView_SPECIMEN_EDITOR;

    public static final String ID = "eu.etaxonomy.taxeditor.editor.view.derivate.DerivateView"; //$NON-NLS-1$

    public static final String YOU_NEED_TO_SAVE_BEFORE_PERFORMING_THIS_ACTION = Messages.DerivateView_YOU_NEED_TO_SAVE;
    public static final String VIEW_HAS_UNSAVED_CHANGES = Messages.DerivateView_UNSAVED_CHANGES;

    private static final List<String> SPECIMEN_INIT_STRATEGY = Arrays.asList(new String[] {
            "descriptions", //$NON-NLS-1$
            "annotations", //$NON-NLS-1$
            "markers", //$NON-NLS-1$
            "credits", //$NON-NLS-1$
            "extensions", //$NON-NLS-1$
            "rights", //$NON-NLS-1$
            "sources", //$NON-NLS-1$
            "derivationEvents.derivatives.annotations", //$NON-NLS-1$
            "derivationEvents.derivatives.markers", //$NON-NLS-1$
            "derivationEvents.derivatives.credits", //$NON-NLS-1$
            "derivationEvents.derivatives.extensions", //$NON-NLS-1$
            "derivationEvents.derivatives.rights", //$NON-NLS-1$
            "derivationEvents.derivatives.sources" //$NON-NLS-1$
    });

	private static final int WARN_THRESHOLD = 200;

    private DelaySelection delaySelection = null;
    /**
     * This is the monitor for the DelaySelection runnable.
     * If it is <code>true</code> then it is currently delaying a selection.
     */
    private boolean isInDelay;


    /**
     * This class invokes internal_selectionChanged() in a separate thread.
     * This allows an asynchronous and/or delayed handling of selection changes
     */
    private class DelaySelection implements Runnable{
        private IWorkbenchPart part;
        private ISelection selection;

        public DelaySelection(IWorkbenchPart part, ISelection selection) {
            super();
            this.part = part;
            this.selection = selection;
        }

        @Override
        public void run() {
            try{
                selectionChanged_internal(part, selection);
            }
            finally{
                isInDelay = false;
            }
        }

        public synchronized void setSelection(ISelection selection) {
            this.selection = selection;
        }

        public synchronized void setPart(IWorkbenchPart part) {
            this.part = part;
        }

    }

	private ConversationHolder conversation;

	private TreeViewer viewer;

    private boolean isDirty;

    private final int dndOperations = DND.DROP_MOVE;

    private DerivateLabelProvider labelProvider;

    private DerivateContentProvider contentProvider;

    private DerivateSearchCompositeController derivateSearchCompositeController;

    /**
     * A map with keys being the derivative entities belonging to the {@link UUID}s passed to the constructor
     * and values being the root elements of the hierarchy (may be the same objects as the derivative entities)
     */
    private Map<SpecimenOrObservationBase<?>, SpecimenOrObservationBase<?>> derivateToRootEntityMap;

    /**
     * The set of root elements
     */
    private Set<SpecimenOrObservationBase<?>> rootElements;

    private ICdmEntitySession cdmEntitySession;

    /**
     * <code>true</code> if this view is listening to selection  changes
     */
    private boolean listenToSelectionChange;

    private Taxon selectedTaxon;

    private ISelectionService selectionService;

    /**
     * Default constructor
     */
    public DerivateView() {
    }


    /**
     * {@inheritDoc}
     */
    @Override
    public void init(IEditorSite site, IEditorInput input) throws PartInitException {
        this.setSite(site);
        this.setInput(input);
        this.derivateToRootEntityMap = new HashMap<SpecimenOrObservationBase<?>, SpecimenOrObservationBase<?>>();
        this.rootElements = new HashSet<SpecimenOrObservationBase<?>>();

        if (CdmStore.isActive() && conversation == null) {
            conversation = CdmStore.createConversation();
        }
        if (CdmStore.isActive()) {
            cdmEntitySession = CdmStore.getCurrentSessionManager().newSession(this, true);
        }
        //listen to context changes
        CdmStore.getContextManager().addContextListener(this);
    }

    @Override
    public void createPartControl(Composite parent) {

        parent.setLayout(new GridLayout());

        //---search and filter---
        derivateSearchCompositeController = new DerivateSearchCompositeController(parent, this);
        GridData gridDataSearchBar = new GridData();
        gridDataSearchBar.horizontalAlignment = GridData.FILL;
        gridDataSearchBar.grabExcessHorizontalSpace = true;
        derivateSearchCompositeController.setLayoutData(gridDataSearchBar);
        derivateSearchCompositeController.setEnabled(CdmStore.isActive());

        //---tree viewer---
        viewer = new TreeViewer(new Tree(parent, SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL | SWT.FULL_SELECTION));
        GridData gridDataTree = new GridData();
        gridDataTree.horizontalAlignment = GridData.FILL;
        gridDataTree.verticalAlignment = GridData.FILL;
        gridDataTree.grabExcessVerticalSpace = true;
        gridDataTree.grabExcessHorizontalSpace = true;
        viewer.getTree().setLayoutData(gridDataTree);
        contentProvider = new DerivateContentProvider();
        viewer.setContentProvider(contentProvider);
        labelProvider = new DerivateLabelProvider();
        labelProvider.setConversation(conversation);
        viewer.setLabelProvider(labelProvider);
        viewer.setAutoExpandLevel(AbstractTreeViewer.ALL_LEVELS);
        viewer.getTree().setEnabled(CdmStore.isActive());
        // Propagate selection from viewer
        getSite().setSelectionProvider(viewer);

        //listen to selection changes
        selectionService = getSite().getWorkbenchWindow().getSelectionService();
        selectionService.addSelectionListener(this);

        //create context menu
        MenuManager menuManager = new MenuManager();
        menuManager.setRemoveAllWhenShown(true);
        getSite().registerContextMenu(menuManager, viewer);
        Control control = viewer.getControl();
        Menu menu = menuManager.createContextMenu(control);
        control.setMenu(menu);

        //init tree
        Collection<UUID> derivativeUuids = ((DerivateViewEditorInput)getEditorInput()).getDerivativeUuids();
        checkWarnThreshold(derivativeUuids);
		updateRootEntities(derivativeUuids);
        //set taxon filter
        derivateSearchCompositeController.setTaxonFilter(((DerivateViewEditorInput) getEditorInput()).getTaxonUuid());
        //reset status bar
        getEditorSite().getActionBars().getStatusLineManager().setMessage(""); //$NON-NLS-1$

        //add drag'n'drop support
        Transfer[] transfers = new Transfer[] {LocalSelectionTransfer.getTransfer(),};
        viewer.addDragSupport(dndOperations, transfers, new DerivateDragListener(this));
        viewer.addDropSupport(dndOperations, transfers, new DerivateDropListener(this));
    }

    public void updateRootEntities() {
        updateRootEntities((Collection)null);
    }

    public void updateRootEntities(Collection<UUID> derivativeUuids) {
        if(conversation!=null){
            if (!conversation.isBound()) {
                conversation.bind();
            }
            /*
             * If the active session is not the session of the Derivative Editor
             * then we will save the active session for later, bind temporarily
             * to our session and rebind to the original session when we are
             * done. This happens e.g. if a selection change happens in the
             * taxon editor and "Link with editor" is enabled. The selection
             * change event and thus the loading in updateRootEntities() happens
             * in the session of the taxon editor.
             */
            ICdmEntitySession previousCdmEntitySession = CdmStore.getCurrentSessionManager().getActiveSession();
            if(cdmEntitySession != null) {
                cdmEntitySession.bind();
            }

            List<SpecimenOrObservationBase> derivates = null;
            if(derivativeUuids!=null){
                this.derivateToRootEntityMap = new HashMap<SpecimenOrObservationBase<?>, SpecimenOrObservationBase<?>>();
                this.rootElements = new HashSet<SpecimenOrObservationBase<?>>();
                derivates = CdmStore.getService(IOccurrenceService.class).load(new ArrayList(derivativeUuids), SPECIMEN_INIT_STRATEGY);
            }
            updateRootEntities(derivates);
            previousCdmEntitySession.bind();
        }
    }


    public void updateRootEntities(List<SpecimenOrObservationBase> derivates) {
            if(derivates!=null){
                this.derivateToRootEntityMap = new HashMap<SpecimenOrObservationBase<?>, SpecimenOrObservationBase<?>>();
                this.rootElements = new HashSet<SpecimenOrObservationBase<?>>();
                for (SpecimenOrObservationBase derivate : derivates) {

                    if(derivate instanceof FieldUnit){
                        derivateToRootEntityMap.put(derivate, derivate);
                    }
                    else {
                        SpecimenOrObservationBase<?> topMostDerivate = EditorUtil.getTopMostDerivate(derivate);
                        if(topMostDerivate!=null){
                            derivateToRootEntityMap.put(derivate, topMostDerivate);
                        }
                        else{
                            derivateToRootEntityMap.put(derivate, derivate);
                        }
                    }
                }
                for (SpecimenOrObservationBase<?> specimen : derivateToRootEntityMap.values()) {
                    rootElements.add(specimen);
                }
            }
            labelProvider.updateLabelCache(rootElements);
            viewer.setInput(rootElements);

            getEditorSite().getActionBars().getStatusLineManager().setMessage(String.format(Messages.DerivateView_CNT_DERIVATIVES_FOUND, rootElements.size()));

            //set selection to derivatives if the filter criteria
            //taxon assignment or derivative type are set
            if(derivates!=null && !derivateSearchCompositeController.isDefaultSearch()){
                List<TreeNode> nodesToSelect = new ArrayList<TreeNode>();
                for (SpecimenOrObservationBase specimenOrObservationBase : derivates) {
                    nodesToSelect.add(new TreeNode(specimenOrObservationBase));
                }
                viewer.setSelection(new StructuredSelection(nodesToSelect));
            }
            else{
                viewer.setSelection(null);
            }

    }

    public void updateLabelCache(){
        labelProvider.updateLabelCache(rootElements);
    }

    @Override
    public void doSave(IProgressMonitor monitor) {
        String taskName = Messages.DerivateView_SAVING_HIERARCHY;
        monitor.beginTask(taskName, 3);
        if (!conversation.isBound()) {
            conversation.bind();
        }
        monitor.worked(1);

        // commit the conversation and start a new transaction immediately
        conversation.commit(true);

        CdmStore.getService(IOccurrenceService.class).merge(new ArrayList<SpecimenOrObservationBase>(rootElements), true);

        monitor.worked(1);

        this.setDirty(false);
        monitor.worked(1);
        monitor.done();
        firePropertyChange(PROP_DIRTY);
        refreshTree();
    }

    @Override
    public void doSaveAs() {
    }

    @Override
    public String getTitleToolTip() {
        return Messages.DerivateView_DERIVATIVE_EDITOR;
    }

    @Override
    public boolean isDirty() {
        return isDirty;
    }

    /**
     * @param isDirty the isDirty to set
     */
    public void setDirty(boolean isDirty) {
        this.isDirty = isDirty;
    }

    @Override
    public boolean isSaveAsAllowed() {
        return false;
    }

    @Override
    public void setFocus() {
        //make sure to bind again if maybe in another view the conversation was unbound
        if(conversation!=null && !conversation.isBound()){
            conversation.bind();
        }
        if(cdmEntitySession != null) {
            cdmEntitySession.bind();
        }
        derivateSearchCompositeController.setFocusOnSearchField();
    }

    @Override
    public void update(CdmDataChangeMap changeEvents) {
    }

    @Override
    public ConversationHolder getConversationHolder() {
        return conversation;
    }

    @Override
    public void changed(Object element) {
        setDirty(true);
        firePropertyChange(IEditorPart.PROP_DIRTY);
        viewer.update(new TreeNode(element), null);
    }

    @Override
    public void forceDirty() {
        changed(null);
    }

    @Override
    public Map<Object, List<String>> getPropertyPathsMap() {
        List<String> specimenPropertyPaths = Arrays.asList(new String[] {
                "descriptions", //$NON-NLS-1$
                "derivationEvents.derivates", //$NON-NLS-1$
                "annotations", //$NON-NLS-1$
                "markers", //$NON-NLS-1$
                "credits", //$NON-NLS-1$
                "extensions", //$NON-NLS-1$
                "rights", //$NON-NLS-1$
                "sources" //$NON-NLS-1$
        });
        Map<Object, List<String>> specimenPropertyPathMap =
                new HashMap<Object, List<String>>();
        specimenPropertyPathMap.put(SpecimenOrObservationBase.class,specimenPropertyPaths);
        return specimenPropertyPathMap;
    }

    /**
     * Refreshes the derivate hierarchy tree and expands the tree
     * to show and select the given object.
     *
     * @param expandTo the object to which the tree should be expanded
     */
    public void refreshTree(Object expandTo){
        refreshTree();
        TreeSelection selection = (TreeSelection) viewer.getSelection();
        viewer.expandToLevel(selection.getFirstElement(), 1);
        viewer.setSelection(new StructuredSelection(new TreeNode(expandTo)));
    }

    /**
     * Refreshes the derivate hierarchy tree
     */
    public void refreshTree(){
    	if(!viewer.getTree().isDisposed()){
    		viewer.refresh();
    	}
    }

    //FIXME:Remoting hack to make this work for remoting
    //This should actually be resolved using remoting post operations
    public void remove(Object obj) {
        rootElements.remove(obj);
        viewer.setInput(rootElements);
    }

    /**
     * @return a set of {@link SingleRead}s that have multiple parents
     */
    public Set<SingleRead> getMultiLinkSingleReads() {
        return DerivateLabelProvider.getMultiLinkSingleReads();
    }

    public Object getSelectionInput() {
        return selectedTaxon;
    }

    public DerivateLabelProvider getLabelProvider() {
        return labelProvider;
    }

    @Override
    public boolean postOperation(CdmBase objectAffectedByOperation) {
        refreshTree();
        if(objectAffectedByOperation!=null){
            changed(objectAffectedByOperation);
        }
        return true;
    }

    @Override
    public boolean onComplete() {
        return true;
    }


    @Override
    public boolean canAttachMedia() {
        return true;
    }

    public void addFieldUnit(FieldUnit fieldUnit) {
        rootElements.add(fieldUnit);
        derivateToRootEntityMap.put(fieldUnit, fieldUnit);
    }

    @Override
    public ICdmEntitySession getCdmEntitySession() {
        return cdmEntitySession;
    }

    @Override
    public void dispose() {
        super.dispose();
        if(conversation!=null){
            conversation.close();
        }
        if(cdmEntitySession != null) {
            cdmEntitySession.dispose();
        }
    }
    public void selectionChanged_internal(IWorkbenchPart part, ISelection selection) {
    	if(part == this){
            return;
        }
        if(viewer.getTree().isDisposed()){
            return;
        }
        if(listenToSelectionChange){
            if(part instanceof MultiPageTaxonEditor){
                selectedTaxon = ((MultiPageTaxonEditor) part).getTaxon();
            }
            else if(selection instanceof IStructuredSelection){
                Object selectedElement = ((IStructuredSelection) selection).getFirstElement();
                if(selectedElement instanceof CdmBase){
                    if(((CdmBase) selectedElement).isInstanceOf(TaxonNode.class)){
                        selectedTaxon = HibernateProxyHelper.deproxy(selectedElement, TaxonNode.class).getTaxon();
                    }
                    else if(((CdmBase) selectedElement).isInstanceOf(Taxon.class)){
                        selectedTaxon = HibernateProxyHelper.deproxy(selectedElement, Taxon.class);
                    }
                }
            }
            if(selectedTaxon!=null){
                Collection<SpecimenOrObservationBase> fieldUnits = CdmStore.getService(IOccurrenceService.class).listFieldUnitsByAssociatedTaxon(selectedTaxon, null, null);
                Collection<UUID> uuids = new HashSet<UUID>();
                for (SpecimenOrObservationBase specimenOrObservationBase : fieldUnits) {
                    uuids.add(specimenOrObservationBase.getUuid());
                }
                checkWarnThreshold(uuids);
                updateRootEntities(uuids);
                setPartName(SPECIMEN_EDITOR+": " + selectedTaxon.getName()); //$NON-NLS-1$
            }
        }
    }

    @Override
    public void selectionChanged(IWorkbenchPart part, ISelection selection) {
    	if(delaySelection==null){
            delaySelection = new DelaySelection(part, selection);
        }
        delaySelection.setPart(part);
        delaySelection.setSelection(selection);
        if(!isInDelay){
            isInDelay = true;
            Display.getCurrent().asyncExec(delaySelection);
        }
    }


	private void checkWarnThreshold(Collection<UUID> uuids) {
		if(uuids!=null && uuids.size()>WARN_THRESHOLD){
			MessagingUtils.warningDialog(Messages.DerivateView_PERF_WARNING, this.getClass(), String.format(Messages.DerivateView_PERF_WARNING_MESSAGE, uuids.size()));
			uuids.clear();
		}
	}

    public TreeViewer getViewer() {
        return viewer;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public List<SpecimenOrObservationBase<?>> getRootEntities() {
        return new ArrayList<SpecimenOrObservationBase<?>>(rootElements);
    }

    public void toggleListenToSelectionChange() {
        listenToSelectionChange = !listenToSelectionChange;
        derivateSearchCompositeController.setEnabled(!listenToSelectionChange);
        if(!listenToSelectionChange){
            selectedTaxon = null;
            setPartName(SPECIMEN_EDITOR);
        }
        else if(selectedTaxon==null){
            setPartName(SPECIMEN_EDITOR+Messages.DerivateView_NO_TAXON_SELECTED);
        }
    }

    public boolean isListenToSelectionChange(){
        return listenToSelectionChange;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void contextAboutToStop(IMemento memento, IProgressMonitor monitor) {
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void contextStop(IMemento memento, IProgressMonitor monitor) {
        derivateSearchCompositeController.setEnabled(false);
        if(!viewer.getTree().isDisposed()) {
            viewer.getTree().setEnabled(false);
            viewer.setInput(null);
        }
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void contextStart(IMemento memento, IProgressMonitor monitor) {
        derivateSearchCompositeController.setEnabled(!listenToSelectionChange);
        if(!viewer.getTree().isDisposed()){
            viewer.getTree().setEnabled(true);
        }
        refreshTree();
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void contextRefresh(IProgressMonitor monitor) {
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void workbenchShutdown(IMemento memento, IProgressMonitor monitor) {
    }

}
