/**
* Copyright (C) 2014 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.editor.view.derivate.operation;

import java.util.UUID;

import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.util.LocalSelectionTransfer;
import org.eclipse.jface.viewers.TreeNode;

import eu.etaxonomy.cdm.api.service.IOccurrenceService;
import eu.etaxonomy.cdm.api.service.molecular.ISequenceService;
import eu.etaxonomy.cdm.model.common.CdmBase;
import eu.etaxonomy.cdm.model.molecular.DnaSample;
import eu.etaxonomy.cdm.model.molecular.Sequence;
import eu.etaxonomy.cdm.model.molecular.SingleRead;
import eu.etaxonomy.cdm.model.occurrence.DerivedUnit;
import eu.etaxonomy.cdm.model.occurrence.SpecimenOrObservationBase;
import eu.etaxonomy.cdm.model.occurrence.SpecimenOrObservationType;
import eu.etaxonomy.taxeditor.editor.internal.TaxeditorEditorPlugin;
import eu.etaxonomy.taxeditor.editor.l10n.Messages;
import eu.etaxonomy.taxeditor.editor.view.derivate.DerivateView;
import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.operation.AbstractPostOperation;
import eu.etaxonomy.taxeditor.operation.IPostOperationEnabled;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author pplitzner
 * @date Nov 18, 2014
 *
 */
public class MoveDerivateOperation extends AbstractPostOperation<CdmBase>  {

    private final TreeNode draggedNode;
    private TreeNode fromNode;
    private TreeNode targetNode;

    public MoveDerivateOperation(String label, IUndoContext undoContext,
            IPostOperationEnabled postOperationEnabled, TreeNode draggedNode, TreeNode targetNode) {
            super(label, undoContext, null, postOperationEnabled);
        this.draggedNode = draggedNode;
        this.targetNode = targetNode;
    }

    @Override
    public IStatus execute(IProgressMonitor monitor, IAdaptable info) throws ExecutionException {
        DerivateView derivateView = null;
        if(getPostOperationEnabled() instanceof DerivateView){
            derivateView = (DerivateView) getPostOperationEnabled();
        }
        if(derivateView==null){
            MessagingUtils.operationDialog(this, new NullPointerException("Derivative Editor was null"), TaxeditorEditorPlugin.PLUGIN_ID, this.getLabel(), getLabel()); //$NON-NLS-1$
            return Status.CANCEL_STATUS;
        }
        if(derivateView.isDirty()){
            MessagingUtils.warningDialog(DerivateView.VIEW_HAS_UNSAVED_CHANGES, this, DerivateView.YOU_NEED_TO_SAVE_BEFORE_PERFORMING_THIS_ACTION);
            return Status.CANCEL_STATUS;
        }

        fromNode = draggedNode.getParent();
        if(moveTreeNode(draggedNode, fromNode, targetNode)){
            //special case if derivative with no parent was moved
            if(fromNode==null && draggedNode!=null && draggedNode.getValue()!=null){
                derivateView.remove(draggedNode.getValue());
            }
            //update DerivateView
            derivateView.getConversationHolder().commit();
            LocalSelectionTransfer.getTransfer().setSelection(null);
            return postExecute(null);
        }
        else{
            MessagingUtils.warningDialog(Messages.MoveDerivateOperation_MOVE_NOT_POSSIBLE, derivateView, String.format(Messages.MoveDerivateOperation_MOVE_TO_NOT_POSSIBLE, derivateView.getLabelProvider().getDerivateText(draggedNode), derivateView.getLabelProvider().getDerivateText(targetNode)));
        }
        return Status.CANCEL_STATUS;
    }

    private boolean moveTreeNode(TreeNode draggedNode, TreeNode fromNode, TreeNode targetNode) {
        Object draggedNodeValue = draggedNode.getValue();
        Object targetNodeValue = targetNode.getValue();
        Object fromParentSpecimen = null;
        if(fromNode!=null){
            fromParentSpecimen = fromNode.getValue();
        }

        // drag'n'drop for SpecimenOrObservationBase
        IOccurrenceService occurrenceService = CdmStore.getService(IOccurrenceService.class);
        if(draggedNodeValue instanceof DerivedUnit && targetNodeValue instanceof SpecimenOrObservationBase<?>){
            DerivedUnit draggedSpecimen = (DerivedUnit) draggedNodeValue;
            SpecimenOrObservationBase<?> targetSpecimen = (SpecimenOrObservationBase<?>) targetNodeValue;
            //check if type is a sub derivate type
            if(fromNode==null || !fromNode.equals(targetNode)){//don't drag on direct parent node)
                UUID fromUuid = null;
                if(fromParentSpecimen!=null){
                    fromUuid = ((SpecimenOrObservationBase<?>)fromParentSpecimen).getUuid();
                }
                return occurrenceService.moveDerivate(fromUuid,
                        targetSpecimen.getUuid(),
                        draggedSpecimen.getUuid()).isOk();
            }
        }

        // drag'n'drop for SingleRead
        else if(draggedNodeValue instanceof SingleRead && targetNodeValue instanceof Sequence){
            SingleRead singleRead = (SingleRead) draggedNodeValue;
            if(fromParentSpecimen instanceof Sequence){
                return CdmStore.getService(ISequenceService.class).moveSingleRead(((Sequence)fromParentSpecimen).getUuid(),
                        ((Sequence)targetNodeValue).getUuid(),
                        singleRead.getUuid()).isOk();

            }
        }

        // drag'n'drop for Sequence
        else if(draggedNodeValue instanceof Sequence && targetNodeValue instanceof DnaSample && ((SpecimenOrObservationBase<?>) targetNodeValue).getRecordBasis()==SpecimenOrObservationType.DnaSample){
            Sequence sequence = (Sequence)draggedNodeValue;
            if(fromParentSpecimen instanceof DnaSample && ((SpecimenOrObservationBase<?>) targetNodeValue).getRecordBasis()==SpecimenOrObservationType.DnaSample){
                return occurrenceService.moveSequence(((DnaSample)fromParentSpecimen).getUuid(),
                        ((DnaSample)targetNodeValue).getUuid(),
                        sequence.getUuid()).isOk();

            }
        }
        return false;
    }

    @Override
    public IStatus redo(IProgressMonitor monitor, IAdaptable info) throws ExecutionException {
        //swap fromNode and targetNode
        this.targetNode = this.fromNode;
        return execute(new NullProgressMonitor(), null);
    }

    @Override
    public IStatus undo(IProgressMonitor monitor, IAdaptable info) throws ExecutionException {
        //swap fromNode and targetNode
        this.targetNode = this.fromNode;
        return execute(new NullProgressMonitor(), null);
    }

}
