/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor.name.operation;

import org.eclipse.core.commands.ExecutionException;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;

import eu.etaxonomy.cdm.model.name.HomotypicalGroup;
import eu.etaxonomy.cdm.model.name.NonViralName;
import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.cdm.model.name.TaxonNameFactory;
import eu.etaxonomy.cdm.model.taxon.SynonymType;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.store.operations.AbstractTaxeditorOperationTestBase;

/**
 * @author n.hoffmann
 * @created 07.04.2009
 */
public class CreateSynonymInExisitingHomotypicalGroupOperationTest extends AbstractTaxeditorOperationTestBase {

	private NonViralName<?> newSynonymName;

	private HomotypicalGroup homotypicalGroup;


	private NonViralName<?> taxonName;

	/**
	 * @throws java.lang.Exception
	 */
	@Before
	public void setUp() throws Exception {

		taxonName = TaxonNameFactory.NewNonViralInstance(null);
		taxon = Taxon.NewInstance(taxonName, null);

		homotypicalGroup = HomotypicalGroup.NewInstance();
		newSynonymName = TaxonNameFactory.NewNonViralInstance(null);

		operation = new CreateSynonymInExistingHomotypicalGroupOperation("Create Synonym In Existing Homotypical Group",
							undoContext, taxon, homotypicalGroup, newSynonymName, postOperation);
	}

	/**
	 * Test method for {@link eu.etaxonomy.taxeditor.editor.name.operation.CreateSynonymInExistingHomotypicalGroupOperation#execute(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)}.
	 *
	 * Homotypic group is the Taxons homotypic group, so we expect the synonym to be homotypic to the accepted taxon.
	 *
	 * @throws ExecutionException
	 */
	@Test
	public void testExecuteTaxonHomotypicGroup() throws ExecutionException {
		homotypicalGroup.addTypifiedName(taxonName);
		operation.execute(monitor, info);

		Assert.assertTrue("Taxon should have a synonym now.", taxon.getSynonyms().size() > 0);
		Assert.assertTrue("Taxon should have a homotypic synonym now", taxon.getHomotypicSynonymsByHomotypicGroup().size() > 0);
	}

	/**
	 * Test method for {@link eu.etaxonomy.taxeditor.editor.name.operation.CreateSynonymInExistingHomotypicalGroupOperation#execute(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)}.
	 *
	 * Homotypic group is not the accepted taxons. Heterotypic synonym should be created.
	 *
	 * @throws ExecutionException
	 */
	@Test
	public void testExecuteSynonymHomotypicGroup() throws ExecutionException {
		// test heterotypic synonym
		operation.execute(monitor, info);

		Assert.assertTrue("Taxon should have a synonym now.", taxon.getSynonyms().size() > 0);
		Assert.assertTrue("Taxon should have a homotypic group", taxon.getHomotypicSynonymyGroups().size() > 0);
		Assert.assertEquals("Synonym relationship should be heterotypic", SynonymType.HETEROTYPIC_SYNONYM_OF(), taxon.getSynonyms().iterator().next().getType());
	}


	/**
	 * Test method for {@link eu.etaxonomy.taxeditor.editor.name.operation.CreateSynonymInExistingHomotypicalGroupOperation#undo(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)}.
	 * @throws ExecutionException
	 */
	@Test
	public void testUndoTaxonHomotypicGroup() throws ExecutionException {
		homotypicalGroup.addTypifiedName(taxonName);

		operation.execute(monitor, info);
		operation.undo(monitor, info);

		Assert.assertTrue("There should not be synonym relationships for the taxon", taxon.getSynonyms().size() == 0);
	}

	/**
	 * Test method for {@link eu.etaxonomy.taxeditor.editor.name.operation.CreateSynonymInExistingHomotypicalGroupOperation#undo(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)}.
	 * @throws ExecutionException
	 */
	@Test
	public void testUndoSynonymHomotypicGroup() throws ExecutionException {
		operation.execute(monitor, info);
		operation.undo(monitor, info);

		Assert.assertTrue("There should not be synonym relationships for the taxon", taxon.getSynonyms().size() == 0);
	}

	/**
	 * Test method for {@link eu.etaxonomy.taxeditor.editor.name.operation.CreateSynonymInExistingHomotypicalGroupOperation#redo(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)}.
	 * @throws ExecutionException
	 */
	@Test
	public void testRedoTaxonHomotypicGroup() throws ExecutionException {
		homotypicalGroup.addTypifiedName(taxonName);

		operation.execute(monitor, info);
		operation.undo(monitor, info);
		operation.redo(monitor, info);

		Assert.assertTrue("Taxon should have a synonym now.", taxon.getSynonyms().size() > 0);
		Assert.assertTrue("Taxon should have a homotypic synonym now", taxon.getHomotypicSynonymsByHomotypicGroup().size() > 0);
	}

	/**
	 * Test method for {@link eu.etaxonomy.taxeditor.editor.name.operation.CreateSynonymInExistingHomotypicalGroupOperation#redo(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)}.
	 * @throws ExecutionException
	 */
	@Test
	public void testRedoSynonymHomotypicGroup() throws ExecutionException {
		operation.execute(monitor, info);
		operation.undo(monitor, info);
		operation.redo(monitor, info);

		Assert.assertTrue("Taxon should have a synonym now.", taxon.getSynonyms().size() > 0);
		Assert.assertTrue("Taxon should have a homotypic group", taxon.getHomotypicSynonymyGroups().size() > 0);
		Assert.assertEquals("Synonym relationship should be heterotypic", SynonymType.HETEROTYPIC_SYNONYM_OF(), taxon.getSynonyms().iterator().next().getType());

	}
}
